package homology.f2homology;

/**
 * This class represents an element of the field with two elements.
 * @author  Patrick Erik Bradley
 * @version 0.1 --- April 9, 2014
 */
public class F2 {
    private Integer number;
        
    /**
     * Constructs an element of F2 from an integer number by taking it modulo 2.
     * @param num an integer number
     */
    public F2(int num) {
	number = num % 2;
    }
    
    /**
     * Returns an integer number corresponding to this element modulo 2.
     */
    public int getNumber() {
	if (number == null) {
	    return 0;
	} else 
	    return number % 2;
    }

    /**
     * Returns the sum of this element and another one.
     * 
     * @param val an F2-value
     * @return sum
     */
    public F2 add(F2 val) {
	if (val.number == (this.number % 2)) return new F2(0);
	return new F2(1);
	}

    /**
     * Returns the product of this element and another one.
     * 
     * @param val
     * @return product
     * 
     */
    public F2 multiply(F2 val) {
	int result = this.number * val.number % 2;
	return new F2(result);
    }

    /**
     * Returns true if this element equals another element.
     * @param val
     * @return true iff the two elements are equal.
     */
    public boolean equals(F2 val) {
	return ((this.number % 2) == (val.number % 2));
    }
	
    /**
     * Returns true if this element equals another object.
     * 
     * @param val
     * @return true iff the two objects are mathematically equal.
     */
    public boolean equals(Object val) {
    	return (this == val);
    }

    /**
     * Returns a String representation of this element.
     * 
     * @return <code>0m2</code> or <code>1m2</code>.
     */
    public String toString() {
	if (this.number == 0 % 2) return "0m2";
	return "1m2";
    }

	
}
