package homology.f2homology;

import java.util.TreeMap;
import java.util.Map;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Set;
import topology.*;

/**
 * This class represents a vector over the field F2 of two elements.
 * @param <K> the index type
 * 
 * @author Patrick Erik Bradley
 * @version 1.0 --- April 9, 2014
 */
@SuppressWarnings("unchecked")
public class F2Vector<K> extends TreeMap<K,F2> {

    private F2[] entries = null;
    private F2 ZERO = new F2(0);
     
    /**
     * Creates a new empty F2Vector.
     */
    public F2Vector() {
    }

    /**
     * Creates an F2Vector from a Map to F2.
     * @param <K> index type
     * @param <F2> the finite field with two elements
     * @param map
     */
    public F2Vector(Map<K,F2> map) {
	for (K k : map.keySet()) {
	    if (map.get(k)!=null) {
		put(k,map.get(k));
	    } else {
		put(k,ZERO);
	    }
	}
    }

    /**
     * Returns the array associated with this F2Vector.
     * @return array
     */
    public F2[] toArray() {
	
	int size = keySet().size();
	entries =  new F2[size];
	int i = -1;
	for (K k : keySet()) {
	    i = i + 1;
	    if (get(k) != null) {
		entries[i] = get(k);
		}
	    else entries[i] = ZERO; 
	}
    	return entries;
    }

    /**
     * Returns the length of this F2Vector.
     * @return length
     */
    public int length() {
	int length = toArray().length;
	return length;
    }

    /**
     * Compares the specified F2Vector with this F2Vector for equality.
     * @param <K> index type
     * @param vector
     * @return true if this F2Vector equals vector
     */
    public boolean equals(F2Vector<K> vector) {
	boolean equals = false;
	if (keySet().equals(vector.keySet()) 
	    && length() == vector.length()) {
	    equals = true;
	} 
	if (equals) {
	    for (K k : keySet()) {
		if (get(k) != null && vector.get(k) != null) {
		    equals = equals && get(k).equals(vector.get(k));
		}
		if (get(k) != null && vector.get(k) == null) {
		    equals = false;
		}
		if (get(k) == null && vector.get(k) != null) {
		    equals = false;
		}
		if (get(k) == null && vector.get(k) == null) {
		    equals = true;
		}
	    } 
	} 
    	return equals;
    } 
}
