package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;
import topology.*;
import homology.*;

public class TestBetti {
    
    FiniteSpace<Integer> emptySpace = TopologyExamples.emptySpace();
    FiniteSpace<Integer> pointSpace = TopologyExamples.pointSpace(5);
    FiniteSpace<Integer> someSpace = TopologyExamples.someSpace();
    FiniteSpace<Integer> circle = TopologyExamples.circle();
    FiniteSpace<Integer> sphere = TopologyExamples.twoSphere();
    FiniteSpace wallet = TopologyExamples.wallet();
    FiniteSpace U3hat = wallet.pNeighbourhood(3);    
    FiniteSpace U3 = wallet.neighbourhood(3);
    FiniteSpace U9 = wallet.closure(9);
    FiniteSpace U9hat = wallet.pClosure(9);
    FiniteSpace<String> mySpace = TopologyExamples.mySpace();

    Betti Betti1 = new Betti(emptySpace);
    Betti Betti2 = new Betti(pointSpace);
    Betti Betti3 = new Betti(someSpace);
    Betti Betti4 = new Betti(circle);
    Betti Betti5 = new Betti(sphere);
    Betti Betti6 = new Betti(wallet);
    Betti Betti7 = new Betti(U3hat);
    Betti Betti8 = new Betti(U3);
    Betti Betti9 = new Betti(U9);
    Betti Betti10 = new Betti(U9hat);
    Betti Betti11 = new Betti(mySpace);
    
    @Before
    public void setUp() {
	
    }
    
    @Test
    public void testF2BettiNumbers() {

	int[] emptyBetti = Betti1.F2BettiNumbers(); 
	assertEquals(emptyBetti[0],0);
	int[] pointBetti = Betti2.F2BettiNumbers();
	assertEquals(pointBetti[0],1);
	int[] someBetti = Betti3.F2BettiNumbers();
	assertEquals(someBetti.length,3);
	assertEquals(someBetti[0],1);
	assertEquals(someBetti[1],0);
	assertEquals(someBetti[2],0);
	int[] circleBetti = Betti4.F2BettiNumbers();
	assertEquals(circleBetti[0],1);
	assertEquals(circleBetti[1],1);
	int[] sphereBetti = Betti5.F2BettiNumbers();
	assertEquals(sphereBetti[0],1);
	assertEquals(sphereBetti[1],0);
	assertEquals(sphereBetti[2],1);
	int[] walletBetti = Betti6.F2BettiNumbers();
	assertEquals(walletBetti[0],1);
	assertEquals(walletBetti[1],0);
	assertEquals(walletBetti[2],0);
	int[] U3hatBetti = Betti7.F2BettiNumbers();
	assertEquals(U3hatBetti[0],1);
	assertEquals(U3hatBetti[1],0);
	int[] U3Betti = Betti8.F2BettiNumbers();
	assertEquals(U3Betti[0],1);
	assertEquals(U3Betti[1],0);
	assertEquals(U3Betti[2],0);
	int[] U9Betti = Betti9.F2BettiNumbers();
	assertEquals(U9Betti[0],1);
	assertEquals(U9Betti[1],0);
	assertEquals(U9Betti[2],0);
	int[] U9hatBetti = Betti10.F2BettiNumbers();
	assertEquals(U9hatBetti[0],1);
	assertEquals(U9hatBetti[1],0);
	int[] myBetti = Betti11.F2BettiNumbers();
	assertEquals(myBetti[0],1);
	assertEquals(myBetti[1],1);
	assertEquals(myBetti[2],0);
    }

    @Test
    public void testF2HomologicalDimension() {
	assertEquals(Betti1.F2HomologicalDimension(),-1);
	assertEquals(Betti2.F2HomologicalDimension(),0);
	assertEquals(Betti3.F2HomologicalDimension(),0);
	assertEquals(Betti4.F2HomologicalDimension(),1);
	assertEquals(Betti5.F2HomologicalDimension(),2);
	assertEquals(Betti6.F2HomologicalDimension(),0);
	assertEquals(Betti7.F2HomologicalDimension(),0);
	assertEquals(Betti8.F2HomologicalDimension(),0);
	assertEquals(Betti9.F2HomologicalDimension(),0);
	assertEquals(Betti10.F2HomologicalDimension(),0);
	assertEquals(Betti11.F2HomologicalDimension(),1);
	//System.out.println(mySpace.dimension());
	assertEquals(mySpace.dimension(),2);
    }
    
    @After
    public void tearDown() {

    }
    
}
