package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;
import topology.*;

public class TestBinaryRelation {
    Set<Pair<Integer,Integer>> myPairs = new TreeSet();
    BinaryRelation<Integer> myRelation = new BinaryRelation();
    BinaryRelation<Integer> yourRelation = new BinaryRelation();
    FiniteSpace<Integer> wallet = TopologyExamples.wallet();


    @Before
    public void setUp() {
    Pair<Integer,Integer> a = new ComparablePair(1,2);
    ComparablePair<Integer,Integer> b = new ComparablePair(1,3);
    ComparablePair<Integer,Integer> c = new ComparablePair(2,2);
    
    myRelation.add(a);   
    myRelation.add(b);
    myRelation.add(c);
    
    myPairs.add(a);
    myPairs.add(b);
    myPairs.add(c);
    yourRelation = new BinaryRelation(myPairs);
    }
    
    @Test
    public void testBinaryRelation() {
	assertEquals(myRelation.getPairs(), yourRelation.getPairs());
    }
    
    @Test
    public void testInLink() {
	Set<Integer> inLink = myRelation.inLink(2);
	Set<Integer> in = new TreeSet();
	in.add(1);
	in.add(2);
	assertEquals(in,inLink);
	Set fiveSix = new TreeSet();
	fiveSix.addAll(Arrays.asList(5,6));
	FiniteSpace<Integer> wallet = TopologyExamples.wallet();
	BinaryRelation rel = wallet.getRelation();
	Set cov_9 = rel.inLink(9);
	assertEquals(cov_9,fiveSix);
	
    }

    @Test
    public void testOutLink() {
	Set<Integer> outLink = myRelation.outLink(1);
	Set<Integer> out = new TreeSet();
	out.add(2);
	out.add(3);
	assertEquals(out,outLink);
    }

    @Test 
    public void testInLinkInWallet() {
	Set<Integer> empty = new TreeSet();
	Set<Integer> in5 = new TreeSet();
	in5.addAll(Arrays.asList(1,2));
	Set<Integer> in6 = new TreeSet();
	in6.addAll(Arrays.asList(1,3));
	Set<Integer> in7 = new TreeSet();
	in7.addAll(Arrays.asList(2,4));
	Set<Integer> in8 = new TreeSet();
	in8.addAll(Arrays.asList(3,4));
	Set<Integer> in9 = new TreeSet();
	in9.addAll(Arrays.asList(5,6));
	Set<Integer> in10 = new TreeSet();
	in10.addAll(Arrays.asList(5,6,7,8));
	Set<Integer> in11 = new TreeSet();
	in11.addAll(Arrays.asList(7,8));
	BinaryRelation<Integer> walletRelation = wallet.getRelation();
	assertEquals(walletRelation.inLink(1),empty);
	assertEquals(walletRelation.inLink(2),empty);
	assertEquals(walletRelation.inLink(3),empty);
	assertEquals(walletRelation.inLink(4),empty);
	assertEquals(walletRelation.inLink(5),in5);
	assertEquals(walletRelation.inLink(6),in6);
	assertEquals(walletRelation.inLink(7),in7);
	assertEquals(walletRelation.inLink(8),in8);
	assertEquals(walletRelation.inLink(9),in9);
	assertEquals(walletRelation.inLink(10),in10);
	assertEquals(walletRelation.inLink(11),in11);
    }

@Test 
    public void testOutLinkInWallet() {
	Set<Integer> empty = new TreeSet();
	Set<Integer> out1 = new TreeSet();
	out1.addAll(Arrays.asList(5,6));
	Set<Integer> out2 = new TreeSet();
	out2.addAll(Arrays.asList(5,7));
	Set<Integer> out3 = new TreeSet();
	out3.addAll(Arrays.asList(6,8));
	Set<Integer> out4 = new TreeSet();
	out4.addAll(Arrays.asList(7,8));
	Set<Integer> out5 = new TreeSet();
	out5.addAll(Arrays.asList(9,10));
	Set<Integer> out6 = new TreeSet();
	out6.addAll(Arrays.asList(9,10));
	Set<Integer> out7 = new TreeSet();
	out7.addAll(Arrays.asList(10,11));
	Set<Integer> out8 = new TreeSet();
	out8.addAll(Arrays.asList(10,11));
	BinaryRelation<Integer> walletRelation = wallet.getRelation();
	assertEquals(walletRelation.outLink(1),out1);
	assertEquals(walletRelation.outLink(2),out2);
	assertEquals(walletRelation.outLink(3),out3);
	assertEquals(walletRelation.outLink(4),out4);
	assertEquals(walletRelation.outLink(5),out5);
	assertEquals(walletRelation.outLink(6),out6);
	assertEquals(walletRelation.outLink(7),out7);
	assertEquals(walletRelation.outLink(8),out8);
	assertEquals(walletRelation.outLink(9),empty);
	assertEquals(walletRelation.outLink(10),empty);
	assertEquals(walletRelation.outLink(11),empty);
    }
    
    
    @Test
    public void testDiagonal() {
	Set<Pair<Integer,Integer>> diagonal = myRelation.diagonal();
	Set<Pair<Integer,Integer>> diag = new TreeSet();
	ComparablePair<Integer,Integer> x = new ComparablePair(1,1);
	ComparablePair<Integer,Integer> y = new ComparablePair(2,2);
	ComparablePair<Integer,Integer> z = new ComparablePair(3,3);
	diag.add(x);
	diag.add(y);
	diag.add(z);
	assertEquals(diag,diagonal);
    }

    @Test
    public void testIsAntiSymmetric() {
	assertEquals(myRelation.isAntiSymmetric(),true);
	ComparablePair<Integer,Integer> d = new ComparablePair(3,1);
	yourRelation.add(d);
	assertEquals(yourRelation.isAntiSymmetric(),false);
    }

    @After
    public void tearDown() {

    }
    
}
