package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;

import topology.*;
import homology.*;
import homology.f2homology.*;

public class TestF2ChainComplex {

    F2 one = new F2(1);
    F2 zero = new F2(0);
    Set<String> emptySet = new TreeSet<String>();
    Set<String> vertices = new TreeSet<String>();
    Set<String> lines = new TreeSet<String>();
    Set<String> areas = new TreeSet<String>();
    F2Matrix<String,String> matrix0 = new F2Matrix<String,String>();
    F2Matrix<String,String> matrix1 = new F2Matrix<String,String>();
    F2Matrix<String,String> matrix2 = new F2Matrix<String,String>();
    TreeMap<ComparablePair<String,String>,F2> map1 
	= new TreeMap<ComparablePair<String,String>,F2>();
    TreeMap<ComparablePair<String,String>,F2> map2 
	= new TreeMap<ComparablePair<String,String>,F2>();
    List<F2Matrix<String,String>> boundaryOperator 
	= new ArrayList<F2Matrix<String,String>>();
    F2ChainComplex<String> chainComplex = new F2ChainComplex<String>();

    @Before
    public void setUp() {
	vertices.addAll(Arrays.asList("0","1","2","3"));
	lines.addAll(Arrays.asList("01","02","12","13","23"));
	areas.add("123");
	
	matrix0 = new F2Matrix(emptySet,vertices);

	map1.put(new ComparablePair("0","01"),one);
	map1.put(new ComparablePair("1","01"),one);
	map1.put(new ComparablePair("0","02"),one);
	map1.put(new ComparablePair("2","02"),one);
	map1.put(new ComparablePair("1","12"),one);
	map1.put(new ComparablePair("2","12"),one);
	map1.put(new ComparablePair("1","13"),one);
	map1.put(new ComparablePair("3","13"),one);
	map1.put(new ComparablePair("2","23"),one);
	map1.put(new ComparablePair("3","23"),one);
	matrix1 = new F2Matrix(map1);
	
	map2.put(new ComparablePair("12","123"),one);
	map2.put(new ComparablePair("13","123"),one);
	map2.put(new ComparablePair("23","123"),one);
	map2.put(new ComparablePair("01","123"),zero);
	map2.put(new ComparablePair("02","123"),zero);
	matrix2 = new F2Matrix(map2);

	boundaryOperator.add(matrix0);
	boundaryOperator.add(matrix1);
	boundaryOperator.add(matrix2);

	chainComplex = new F2ChainComplex(boundaryOperator);
    }
      
    @Test
    public void testGetBoundaryOperator() {
	List<F2Matrix<String,String>> boundary 
	    = chainComplex.getBoundaryOperator();
	//for (int i = 0; i < boundary.size(); i++) {
	//    System.out.println(boundary.get(i));
	//}
	//System.out.println();
	
	//for (int i = 0; i < 5; i++) {
	//    for (int j = 0; j < 5; j++) {
	//	System.out.print(boundary.get(1).readOffNF().get(i)[j] + " ");
	//   }
	//  System.out.println();
	//}
	boolean b0 = boundary.get(0).equals(matrix0);
	assertEquals(b0,true);
	boolean b1 = boundary.get(1).equals(matrix1);
	assertEquals(b1,true);
	boolean b2 = boundary.get(2).equals(matrix2);
	assertEquals(b2,true);
		
    }

    @Test
    public void testGetDimension() {
	F2Matrix b = matrix1.product(matrix2);
	F2Matrix<String,String> zero12 = new F2Matrix();
	TreeMap<ComparablePair<String,String>,F2> map12 = new TreeMap();
	map12.put(new ComparablePair("0","123"),zero);
	map12.put(new ComparablePair("1","123"),zero);
	map12.put(new ComparablePair("2","123"),zero);
	map12.put(new ComparablePair("3","123"),zero);
	zero12 = new F2Matrix(map12);
	assertEquals(b.equals(zero12),true);
	
	F2Matrix<String,String> zeroMatrix12 = new F2Matrix();
	zeroMatrix12 =  new F2Matrix(matrix1.rows(),matrix2.columns(),zero);
	
	assertEquals(b.rowList().equals(zeroMatrix12.rowList()),true);	
	assertEquals(b.equals(zeroMatrix12),true);
	assertEquals(zero12.equals(zeroMatrix12),true);
	assertEquals(zero12.equals(zero12),true);
	assertEquals(chainComplex.getDimension(),2);
    }

    @Test
    public void testBettiNumbers() {
	int[] betti = chainComplex.BettiNumbers();
	assertEquals(betti.length,3);
	assertEquals(chainComplex.BettiNumbers()[0],1);
	assertEquals(chainComplex.BettiNumbers()[1],1);
	assertEquals(chainComplex.BettiNumbers()[2],0);
    }
    
    @Test
    public void testHomologicalDimension() {
	assertEquals(chainComplex.homologicalDimension(),1);
    }

    @After
    public void tearDown() {
    }
    

}
