package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;

import topology.*;
import homology.*;
import homology.f2homology.*;

public class TestF2Module {

    F2 one = new F2(1);
    F2 zero = new F2(0);
    F2Matrix matrix = new F2Matrix();
    TreeMap<ComparablePair<Integer,Integer>,F2> map = new TreeMap();
    F2Vector c1 = new F2Vector();
    TreeMap c1Map = new TreeMap();
    F2Vector c2 = new F2Vector();
    TreeMap c2Map = new TreeMap();
    F2Vector c3 = new F2Vector();
    TreeMap c3Map = new TreeMap();
    F2Vector c4 = new F2Vector();
    TreeMap c4Map = new TreeMap();
    List<F2Vector> moduleList = new ArrayList();
    F2Module module = new F2Module();
    
    @Before
    public void setUp() {
	map.put(new ComparablePair(1,1),zero);
	map.put(new ComparablePair(1,3),one);
	map.put(new ComparablePair(2,0),one);
	map.put(new ComparablePair(2,2),one);
	map.put(new ComparablePair(3,0),one);
	map.put(new ComparablePair(3,3),one);
	matrix = new F2Matrix(map);
	
	c1Map.put(1,zero); c1Map.put(2,one); c1Map.put(3,one);
	c1 = new F2Vector(c1Map);
	c2Map.put(1,zero); c2Map.put(2,zero); c2Map.put(3,zero);
	c2 = new F2Vector(c2Map);
	c3Map.put(1,zero); c3Map.put(2,one); c3Map.put(3,zero);
	c3 = new F2Vector(c3Map);
	c4Map.put(1,one); c4Map.put(2,zero); c4Map.put(3,one);
	c4 = new F2Vector(c4Map);

	moduleList.add(c1); 
	moduleList.add(c2); 
	moduleList.add(c3);
	moduleList.add(c4);
	
	module = new F2Module(moduleList);
    }
    
    @Test
    public void testGetGeneratingSystem() {
	List<F2Vector> system = module.getGeneratingSystem();
	assertEquals(system,moduleList);
    }

    @Test
    public void testGetMatrix() {
	F2Matrix M = module.getMatrix();
	assertEquals(M.equals(matrix),true);
	assertEquals(M.rows().equals(matrix.rows()),true);
	assertEquals(M.columns().equals(matrix.columns()),true);
    }

    @Test
    public void testBasis() {
	List<F2Vector> L = module.basis();
	moduleList.remove(c2);
	assertEquals(L.equals(moduleList),true);
    }
   
    @Test
    public void testGetRank() {
	assertEquals(module.getRank(),3);
    }
    
    @After
    public void tearDown() {

    }
    

}
