package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;
import topology.*;
import homology.*;
import homology.f2homology.*;

public class TestF2OrderComplexQuotient {
    
    FiniteSpace<String> mySpace = TopologyExamples.mySpace();
    FiniteSpace<String> yas = TopologyExamples.yas();
    PrimMST mySpaceMst = new PrimMST(mySpace);
    PrimMST yasMst = new PrimMST(yas);
    Betti Betti1 = new Betti(mySpace);
    Betti Betti2 = new Betti(yas);
    F2OrderComplexQuotient<Integer> mySpaceQuotient 
	= new F2OrderComplexQuotient(mySpaceMst.getMst(),mySpace);    
    F2OrderComplexQuotient yasQuotient
	= new F2OrderComplexQuotient(yasMst.getMst(),yas);

    Set<Integer> spacePoints = new TreeSet();
    BinaryRelation<Integer> spaceRelation = new BinaryRelation();
    FiniteSpace<Integer> space = new FiniteSpace();
    FiniteSpace emptySpace = new FiniteSpace();
    PrimMST emptyMst = new PrimMST(emptySpace);
    F2OrderComplexQuotient emptyQuotient
	= new F2OrderComplexQuotient(emptyMst.getMst(),emptySpace);
    PrimMST spaceMst = new PrimMST();
    Betti Betti3 = new Betti();
    F2OrderComplexQuotient spaceQuotient = new F2OrderComplexQuotient();
    

    @Before
    public void setUp() {
	spacePoints.addAll(Arrays.asList(1,2,3,4,5,6,7));
	spaceRelation.add(new ComparablePair(1,2));
	spaceRelation.add(new ComparablePair(2,4));
	spaceRelation.add(new ComparablePair(1,4));
	spaceRelation.add(new ComparablePair(3,4));
	spaceRelation.add(new ComparablePair(5,6));
	space = new FiniteSpace(spacePoints,spaceRelation);
	spaceMst = new PrimMST(space);	
	spaceQuotient = new F2OrderComplexQuotient(spaceMst.getMst(),space);
	
    }
    
    @Test
    public void testF2ChainComplex() {
	F2ChainComplex mySpaceQuotientComplex = mySpaceQuotient.getF2ChainComplex();
	int[] mySpaceQuotientBetti = mySpaceQuotientComplex.BettiNumbers();
	assertEquals(mySpaceQuotientBetti[0],0);
	F2ChainComplex yasQuotientComplex = yasQuotient.getF2ChainComplex();
	int[] yasQuotientBetti = yasQuotientComplex.BettiNumbers();
	assertEquals(yasQuotientBetti[0],0);
	int[] betti1 = Betti1.F2BettiNumbers();
	for (int i = 1; i < betti1.length; i++) {
	    assertEquals(mySpaceQuotientBetti[i], betti1[i]);
	}
	int[] betti2 = Betti2.F2BettiNumbers();
	for (int i = 1; i < betti2.length; i++) {
	    assertEquals(yasQuotientBetti[i],betti2[i]);
	}
    }    
    
    @Test
    public void testIsSpecialSubmersion() {
	assertEquals(spaceQuotient.isSpecialSubmersion(),false);
    }

    @After
    public void tearDown() {

    }
    
}
