package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;

import topology.*;
import homology.*;
import homology.f2homology.*;

public class TestOrderComplex {
    
    FiniteSpace<String> mySpace = new FiniteSpace<String>();
    Set<String> myPointSet = new TreeSet<String>();
    BinaryRelation<String> myRelation = new BinaryRelation<String>();
    Set<String> emptySet = new TreeSet<String>();
    Set<String> vertices = new TreeSet<String>();
    Set<String> lines = new TreeSet<String>();
    Set<String> areas = new TreeSet<String>();
    OrderComplex<String> orderComplex = new OrderComplex<String>();

    @Before
    public void setUp() {

	vertices.addAll(Arrays.asList("0","1","2","3"));
	lines.addAll(Arrays.asList("01","02","12","13","23"));
	areas.add("123");

	myPointSet.addAll(vertices);
	myPointSet.addAll(lines);
	myPointSet.addAll(areas);

	myRelation.add(new ComparablePair("123","12"));
	myRelation.add(new ComparablePair("123","13"));
	myRelation.add(new ComparablePair("123","23"));
	myRelation.add(new ComparablePair("12","1"));
	myRelation.add(new ComparablePair("12","2"));
	myRelation.add(new ComparablePair("13","1"));
	myRelation.add(new ComparablePair("13","3"));
	myRelation.add(new ComparablePair("23","2"));
	myRelation.add(new ComparablePair("23","3"));
	myRelation.add(new ComparablePair("01","0"));
	myRelation.add(new ComparablePair("01","1"));
	myRelation.add(new ComparablePair("02","0"));
	myRelation.add(new ComparablePair("02","2"));

	mySpace = new FiniteSpace(myPointSet,myRelation);
	
	orderComplex = new OrderComplex(mySpace);
    }
      
    @Test
    public void testGetMaximalChains() {
	TreeSet<List<String>> maxChains = orderComplex.getMaximalChains();
	assertEquals(maxChains.size(),10);
    }

    @Test
    public void testDimension() {
	assertEquals(orderComplex.dimension(),2);
	
    }

    @Test
    public void testMaximalSimplices() {
	Set<Set<String>> maxSimp = orderComplex.maximalSimplices();
	assertEquals(maxSimp.size(),10);
	Set<String> simplex = new TreeSet();
	simplex.addAll(Arrays.asList("1","12","123"));
	assertEquals(maxSimp.contains(simplex),true);
    }

    @Test
    public void testSimplices() {
	Set<Set<String>> Simplices = orderComplex.simplices();
	//System.out.println(Simplices);
	assertEquals(Simplices.size(),32);
    }

    @Test
    public void testListOfSimplexSets() {
	List<TreeSet<Integer>> simpList = orderComplex.listOfSimplexSets();
	assertEquals(simpList.get(0).size(),10);
	//System.out.println(simpList.get(0));
	assertEquals(simpList.get(1).size(),16);
	//System.out.println(simpList.get(1));
	assertEquals(simpList.get(2).size(),6);
	//System.out.println(simpList.get(2));
    }

    @Test
    public void testInverseMap() {
	TreeMap<Set<String>,Integer> indexMap = orderComplex.indexMap();
	TreeMap<Integer,Set<String>> inverseMap = orderComplex.inverseMap();
	int N = orderComplex.simplices().size();
	for (int i = 0; i < N; i++) {
	    Set<String> simplex = inverseMap.get(i);
	    Integer index = indexMap.get(simplex);
	    boolean inverse = index.equals(i);
	    //System.out.print(index);
	    //System.out.println(simplex);
	    assertEquals(inverse,true);
	}
    }

    @Test
    public void testF2ChainComplex() {
	F2ChainComplex<Integer> myChainComplex = orderComplex.F2ChainComplex();
	TreeMap<Integer,Set<String>> inverseMap = orderComplex.inverseMap();
	List<F2Matrix<Integer,Integer>> myBd = myChainComplex.getBoundaryOperator();
	//int t = 0;
	//for (Integer i : myBd.get(1).rows()) {
	//    for (Integer j : myBd.get(1).columns()) {
	//	if (myBd.get(1).get(new ComparablePair(i,j)).equals(one)) {
	//	    System.out.print(i + " ");
	//	    System.out.print(j + " ");
	//	    System.out.println(myBd.get(1)
	//			       .get(new ComparablePair(i,j)));
	//	    t = t + 1;
	//	}
	//   }
	//}
	//System.out.println(t);
	//System.out.println(myBd.get(1).rows());
	//System.out.println(myBd.get(1).columns());
	//t = 0;
	//for (Integer i : myBd.get(2).rows()) {
	//    for (Integer j : myBd.get(2).columns()) {
	//	if (myBd.get(2).get(new ComparablePair(i,j)).equals(one)) {
	//	    System.out.print(i + " ");
	//	    System.out.print(j + " ");
	//	    System.out.println(myBd.get(2)
	//			       .get(new ComparablePair(i,j)));
	//	    t++;
	//	}
	//    }
	//}
	//System.out.println(t);

    }

    @After
    public void tearDown() {
    }
    

}
