package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;
import topology.*;

public class TestPointReductions {
    
    Set emptySet = new TreeSet();
    FiniteSpace<Integer> emptySpace = TopologyExamples.emptySpace();
    FiniteSpace<Integer> pointSpace = TopologyExamples.pointSpace(5);
    FiniteSpace<Integer> someSpace = TopologyExamples.someSpace();
    FiniteSpace<Integer> circle = TopologyExamples.circle();
    FiniteSpace<Integer> sphere = TopologyExamples.twoSphere();
    FiniteSpace wallet = TopologyExamples.wallet();
    FiniteSpace U3hat = new FiniteSpace(wallet.pNeighbourhood(3));  
    FiniteSpace U3 = wallet.neighbourhood(3);
    FiniteSpace U9 = wallet.closure(9);
    FiniteSpace U9hat = wallet.pClosure(9);

    @Before
    public void setUp() {
	    
    }

    @Test
    public void testIsUpBeatPoint() {
	boolean someUb4 = PointReductions.isUpBeatPoint(someSpace,4);
	boolean someUb5 = PointReductions.isUpBeatPoint(someSpace,5);
	boolean someUb1 = PointReductions.isUpBeatPoint(someSpace,1);
	assertEquals(someUb4,true);
	assertEquals(someUb5,true);
	assertEquals(someUb1,false);
    }
    
   
    @Test
    public void testIsDownBeatPoint() {
	boolean someDb4 = PointReductions.isDownBeatPoint(someSpace,4);
	boolean someDb5 = PointReductions.isDownBeatPoint(someSpace,5);
	boolean someDb1 = PointReductions.isDownBeatPoint(someSpace,1);
	assertEquals(someDb4,true);
	assertEquals(someDb5,false);
	assertEquals(someDb1,true);
    }
    
    @Test
    public void testUpBeatPoints() {
	Set nineEleven = new TreeSet();
	nineEleven.addAll(Arrays.asList(9,11));
	Set fiveSix = new TreeSet();
	fiveSix.addAll(Arrays.asList(5,6));
	BinaryRelation rel = wallet.getRelation();
	Set cov_9 = rel.inLink(9);
	//System.out.println(cov_9);
	assertEquals(cov_9,fiveSix);
	BinaryRelation relation = new BinaryRelation(wallet.HasseDiagram());
	Set covering_9 = relation.inLink(9);
	assertEquals(covering_9,fiveSix);
	Set six89Eleven = new TreeSet();
	six89Eleven.addAll(Arrays.asList(6,8,9,11));
	
	assertEquals(PointReductions.upBeatPoints(wallet),emptySet);
	assertEquals(PointReductions.upBeatPoints(U3hat),nineEleven);
	assertEquals(PointReductions.upBeatPoints(U3),six89Eleven);
	
    }

    @Test
    public void testDownBeatPoints() {
	Set three256 = new TreeSet();
	three256.addAll(Arrays.asList(2,3,5,6));
	Set three2 = new TreeSet();
	three2.addAll(Arrays.asList(2,3));
	
	assertEquals(PointReductions.downBeatPoints(wallet),emptySet);
	assertEquals(PointReductions.downBeatPoints(U9),three256);
	assertEquals(PointReductions.downBeatPoints(U9hat),three2);
    }

    @Test
    public void testNoBeatPoint() {
	assertEquals(PointReductions.noBeatPoint(emptySpace),true);
	assertEquals(PointReductions.noBeatPoint(someSpace),false);
	assertEquals(PointReductions.noBeatPoint(pointSpace),true);

	assertEquals(PointReductions.noBeatPoint(circle),true);
	assertEquals(PointReductions.noBeatPoint(sphere),true);
		
	assertEquals(PointReductions.noBeatPoint(wallet),true);
	assertEquals(PointReductions.noBeatPoint(U3hat),false);
	
    }
    
    @Test
    public void testRemoveBeatPoints() {
	assertEquals(PointReductions.removeBeatPoints(someSpace).getPointSet(),
		     pointSpace.getPointSet());
	assertEquals(PointReductions.removeBeatPoints(someSpace).getRelation().getPairs(), pointSpace.getRelation().getPairs());
	
	FiniteSpace<Integer> circ = PointReductions.removeBeatPoints(circle);
	assertEquals(circle.getPointSet(),circ.getPointSet());
	assertEquals(circle.getRelation().getPairs(),
		     circ.getRelation().getPairs());
	
	FiniteSpace<Integer> core = PointReductions.removeBeatPoints(sphere);
	assertEquals(core.getPointSet(),sphere.getPointSet());
	assertEquals(core.getRelation().getPairs(),
		     sphere.getRelation().getPairs());

	FiniteSpace<Integer> core2 = PointReductions.removeBeatPoints(wallet);
	assertEquals(core2.getPointSet(),wallet.getPointSet());
	assertEquals(core2.getRelation().getPairs(),
		     wallet.getRelation().getPairs());
	//System.out.println(wallet.dimension());
	assertEquals(wallet.dimension(),2);

	FiniteSpace<Integer> pU2 = wallet.pNeighbourhood(2);
	FiniteSpace<Integer> pt2 = PointReductions.removeBeatPoints(pU2);
	assertEquals(pt2.getPointSet().size(),1);
	
	FiniteSpace<Integer> pU3 = wallet.pNeighbourhood(3);
	FiniteSpace<Integer> pt3 = PointReductions.removeBeatPoints(pU3);
	assertEquals(pt3.getPointSet().size(),1);
    }

    @Test
    public void testIsUpWeakPoint() {
	boolean someUw4 = PointReductions.isUpWeakPoint(someSpace,4);
	assertEquals(someUw4,true);
	boolean walletUw3 = PointReductions.isUpWeakPoint(wallet,3);
	assertEquals(walletUw3,true);
	boolean walletUw11 = PointReductions.isUpWeakPoint(wallet,11);
	assertEquals(walletUw11,false);
    }

    @Test
    public void testIsDownWeakPoint() {
	boolean someDw4 = PointReductions.isDownWeakPoint(someSpace,4);
	assertEquals(someDw4,true);
	boolean walletDw3 = PointReductions.isDownWeakPoint(wallet,3);
	assertEquals(walletDw3,false);
	boolean walletDw11 = PointReductions.isDownWeakPoint(wallet,11);
	assertEquals(walletDw11,true);
    }
     
    @Test
    public void testNoWeakPoint() {
	assertEquals(PointReductions.noWeakPoint(someSpace),false);
	assertEquals(PointReductions.noWeakPoint(circle),true);
	assertEquals(PointReductions.noWeakPoint(wallet),false);
	       
    }

    @Test
    public void testRemoveWeakPoints() {
	FiniteSpace<Integer> w = PointReductions.removeWeakPoints(wallet);
	assertEquals(w.getPointSet().size(),1);
	//System.out.println(wallet.dimension());
	assertEquals(wallet.dimension(),2);

	FiniteSpace<Integer> circ = PointReductions.removeWeakPoints(circle);
	assertEquals(circ.getPointSet(),circle.getPointSet());
	assertEquals(circ.getRelation().getPairs(),
		     circle.getRelation().getPairs());

	FiniteSpace<Integer> s = PointReductions.removeWeakPoints(someSpace);
	assertEquals(s.getPointSet(),pointSpace.getPointSet());
	assertEquals(s.getRelation().getPairs(),
		     pointSpace.getRelation().getPairs());
    }
    
    @After
    public void tearDown() {

    }
    
}
