package tests;

import org.junit.*;
import static org.junit.Assert.*;
import java.util.*;
import topology.*;
import homology.*;

public class TestPrimMST {
    
    FiniteSpace<String> mySpace = TopologyExamples.mySpace();
    FiniteSpace<String> yas = TopologyExamples.yas();
    PrimMST mySpaceMst = new PrimMST(mySpace);
    PrimMST yasMst = new PrimMST(yas);
    Betti Betti1 = new Betti(mySpaceMst.getMst());
    Betti Betti2 = new Betti(yasMst.getMst());
    
    Set<Integer> spacePoints = new TreeSet();
    BinaryRelation<Integer> spaceRelation = new BinaryRelation();
    FiniteSpace<Integer> space = new FiniteSpace();
    FiniteSpace emptySpace = new FiniteSpace();
    PrimMST emptyMst = new PrimMST(emptySpace);
    PrimMST spaceMst = new PrimMST();
    Betti Betti3 = new Betti();
    Betti Betti4 = new Betti();

    @Before
    public void setUp() {
	spacePoints.addAll(Arrays.asList(1,2,3,4,5,6,7));
	spaceRelation.add(new ComparablePair(1,2));
	spaceRelation.add(new ComparablePair(2,4));
	spaceRelation.add(new ComparablePair(1,4));
	spaceRelation.add(new ComparablePair(3,4));
	spaceRelation.add(new ComparablePair(5,6));
	space = new FiniteSpace(spacePoints,spaceRelation);
	spaceMst = new PrimMST(space);	
	Betti3 = new Betti(spaceMst.getMst());
	Betti4 = new Betti(space);
    }
    
    @Test
    public void testGetMst() {
	int[] betti1 = Betti1.F2BettiNumbers();
	int[] betti2 = Betti2.F2BettiNumbers();
	int[] betti3 = Betti3.F2BettiNumbers();
	int[] betti4 = Betti4.F2BettiNumbers();
	assertEquals(betti1[0],1); 
	assertEquals(betti2[0],1);
	assertEquals(betti3[0],3);
	assertEquals(betti3[0],betti4[0]);
	for (int i = 1; i < betti1.length; i++) {
	    	assertEquals(betti1[i],0);
	}
	for (int i = 1; i < betti2.length; i++) {
	    assertEquals(betti2[i],0);
	}
	for (int i = 1; i < betti3.length; i++) {
	    assertEquals(betti3[i],0);
	}
	
	
    }

    @Test
    public void testComponentNumber() {
	assertEquals(mySpaceMst.componentNumber(),1);
	assertEquals(yasMst.componentNumber(),1);
    }
    
    @After
    public void tearDown() {

    }
    
}
