package topology;

import java.util.*;

/**
 * The class ComparablePair defines pairs of Comparable objects. 
 * These pairs are made comparable by the lexicographic order.
 * @author Patrick Erik Bradley
 * @version 1.0
 */

public class ComparablePair<K extends Comparable<K>,V extends Comparable<V>> 
    implements Comparable<Pair<K,V>>, Pair<K,V> {
    
    private K key;
    private V value;
    
    /**
     * Initialises a ComparablePair.
     */
    public ComparablePair(){
    }
    
    /**
     * Creates a ComparablePair with given key and value.
     * @param <K> key type
     * @param <V> value type
     * @param key
     * @param value
     */
    public ComparablePair(K key, V value){
	this.key = key;
	this.value = value;
    }

    /**
     * Returns the key (= first element) of a ComparablePair.
     * @return key
     */
    public K getKey()	{ 
	return key; 
    }

    /**
     * Returns the value (= second element) of a ComparablePair.
     * @return value
     */
    public V getValue() { 
	return value; 
    }

    /**
     * Compares a ComparablePair with a pair. 
     * Returns 1 if it is lexicographically larger than the pair, 
     * -1 if it is lexicographically less than the pair, 
     * and 0 if it equals the pair. 
     * The lexicographic order is consistent with equals.
     * @param <K> key type
     * @param <V> value
     * @param a a pair
     * @return compareTo-value
     */
    @Override
    public int compareTo(Pair<K,V> a){
	int comp;
	K aKey = a.getKey();
	V aValue = a.getValue();
	
	int compKey = key.compareTo(aKey);
	int compVal = value.compareTo(aValue);
	comp = compKey;
	if (comp == 0) {
	    comp = compVal;
	}
	return comp;
    }

    /**
     * Returns a string representation of this ComparablePair.
     * @return a string representation of this ComparablePair
     */
    public String toString() {
	return "(" + getKey() + "," + getValue() + ")";
    }
}
