package topology;

import java.util.*;

/**
 * This class provides for a comparator of lists. They are compared with
 * respect to lexicographical ordering. 
 *
 * @author Patrick Erik Bradley and Norbert Paul
 */
public class ListComparator<E extends Comparable<E>> implements Comparator<List<E>> {

    /**
     * Returns the compare-value of two lists with respect to 
     * lexicographical ordering: a positive number if the second list is 
     * before the first list, a negative number if the first list is before
     * the second list, and zero if both lists are the same.
     * @param <E> list type
     * @param list1
     * @param list2
     * @return compare-value
     */
    @Override
	public int compare(List<E> list1, List<E> list2) {
	int comp;
	int n = Math.min(list1.size(),list2.size());
	for (int i = 0; i < n; i++) {
	    E e1 = list1.get(i);
	    E e2 = list2.get(i);
	    int result = e1.compareTo(e2);
	    if (result != 0) {
		return result;
	    }
	}
	return list1.size() - list2.size();
    }
}
