package topology;

import java.util.*;
import java.lang.*;

/**
 * The class TopoCollection consists of some methods for 
 * finite topological spaces.
 * @author Patrick Erik Bradley
 * @version 1.0
 */
public class TopoCollection {
   
    /**
     * Returns the non-Hausdorff join of two finite topological spaces
     * of Integer type.
     * @param space1 finite space 1
     * @param space2 finite space 2
     * @return the non-Hausdorff join
     */
    public static FiniteSpace<Integer> 
			     nonHausdorffJoin(FiniteSpace<Integer> space1, 
					      FiniteSpace<Integer> space2) {
	FiniteSpace<Integer> result = new FiniteSpace();
	Set<Integer> set1 = new TreeSet();
	Set<Integer> set2 = new TreeSet();
	BinaryRelation<Integer> rel1 = new BinaryRelation();
	BinaryRelation<Integer> rel2 = new BinaryRelation();	
	Set<Integer> pointSet1 = space1.getPointSet();
	Set<Integer> pointSet2 = space2.getPointSet();
	Set<Integer> intersection = new TreeSet(pointSet1);
	intersection.retainAll(pointSet2);
	if (intersection.size() == 0) {
	    set1 = pointSet1;
	    set2 = pointSet2;
	    rel1 = space1.getRelation();
	    rel2 = space2.getRelation();
	}
	else {
	    set1 = pointSet1;
	    rel1 = space1.getRelation();
	    int min1 = 0;
	    int max1 = 0;
	    for (int e1 : pointSet1) {
		max1 = e1;
		min1 = e1;
		break;
	    }
	    for (int e1 : pointSet1) {
		max1 = Math.max(e1,max1);
		min1 = Math.min(e1,min1);
	    }
	    int width1 = max1 - min1;
	    for (int e2 : pointSet2) {
		set2.add(e2 + width1 + 1);
	    }
	    Iterator<Pair<Integer,Integer>> iter 
		= space2.getRelation().getPairs().iterator();
	    while (iter.hasNext()) {
		Pair<Integer,Integer> e = iter.next();
		Integer keyNew = e.getKey() + width1 + 1;
		Integer valueNew = e.getValue() + width1 + 1;
		Pair<Integer,Integer> pair 
		    = new ComparablePair(keyNew,valueNew);
		rel2.add(pair);
	    }
	}
	result.addAllPoint(set1);
	result.addAllPoint(set2);
	result.addAllPair(rel1.getPairs());
	result.addAllPair(rel2.getPairs());
	
	Iterator<Integer> iter1 = set1.iterator();
	while (iter1.hasNext()) {
	    Integer e1 = iter1.next();
	    Iterator<Integer> iter2 = set2.iterator();
	    while (iter2.hasNext()) {
		Integer e2 = iter2.next();
		result.addPair(new ComparablePair(e1,e2));
	    }
	}
	return result;
    }
    
}
