package topology;

import java.util.*;

/**
This interface models finite topological spaces.
@author Patrick Erik Bradley
@version 1.0
 */
public interface TopologicalSpace<E>  {

    /**
     * Returns the point set underlying the topological space.
     * @param<E> space type
     * @return point set
     */
    public Set<E> getPointSet();


    /**
     * Returns the relation which defines the topology. 
     * @return relation
     */
    public BinaryRelation getRelation();
    
    /** 
     * Returns true if the space is T0-separable.
     * @return true if space is T0-separable
     */
    public boolean isTzero();
}
