package topology;

import java.util.*;

/**
 * This class provides for a comparator of TreeSets. They are compared with
 * respect to lexicographical ordering. 
 *
 * @author Patrick Erik Bradley and Norbert Paul
 */
public class TreeSetComparator<E extends Comparable<E>> implements Comparator<TreeSet<E>> {

    /**
     * Returns the compare-value of two TreeSets with respect to lexicographical
     * ordering: a positive value if the second tree set is before the first
     * tree set, a negative value if the first tree set is before the second
     * tree set, and zero if both are the same.
     * @param <E> tree set type
     * @param treeSet1 tree set 1
     * @param treeSet2 tree set 2
     * @return compare-value
     */ 
    @Override
	public int compare(TreeSet<E> treeSet1, TreeSet<E> treeSet2) {
	    TreeSet<E> set1 = (TreeSet<E>)treeSet1.clone();
	    TreeSet<E> set2 = (TreeSet<E>)treeSet2.clone();
	    int n = Math.min(set1.size(),set2.size());
	    for (int i = 0; i < n; i++) {
		E e1 = set1.pollFirst();
		E e2 = set2.pollFirst();
		int result = e1.compareTo(e2);
		if (result != 0) {
		    return result;
		}
	    }
	    return set1.size() - set2.size();
    }
}
